@@ -1,6 +1,6 @@
 /* Copyright (C) 2001-2007 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
@@ -11,7 +11,7 @@
    San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gsstruct.h,v 1.2 2010/07/10 22:02:25 Arabidopsis Exp $ */
+/* $Id$ */
 /* Definitions for Ghostscript modules that define allocatable structures */
 /* Requires gstypes.h */
 
@@ -144,7 +144,7 @@ struct gs_gc_root_s {
 /* Print a root debugging message. */
 #define if_debug_root(c, msg, rp)\
   if_debug4(c, "%s 0x%lx: 0x%lx -> 0x%lx\n",\
-	    msg, (ulong)(rp), (ulong)(rp)->p, (ulong)*(rp)->p)
+            msg, (ulong)(rp), (ulong)(rp)->p, (ulong)*(rp)->p)
 
 /*
  * We don't want to tie the allocator to using a single garbage collector,
@@ -163,13 +163,13 @@ struct gs_gc_root_s {
 #define param_string_proc_reloc(proc)\
   void proc(gs_param_string *, gc_state_t *)
 #define gc_procs_common\
-	/* Relocate a pointer to an object. */\
+        /* Relocate a pointer to an object. */\
   ptr_proc_reloc((*reloc_struct_ptr), void /*obj_header_t*/);\
-	/* Relocate a pointer to a string. */\
+        /* Relocate a pointer to a string. */\
   string_proc_reloc((*reloc_string));\
-	/* Relocate a pointer to a const string. */\
+        /* Relocate a pointer to a const string. */\
   const_string_proc_reloc((*reloc_const_string));\
-	/* Relocate a pointer to a parameter string. */\
+        /* Relocate a pointer to a parameter string. */\
   param_string_proc_reloc((*reloc_param_string))
 typedef struct gc_procs_common_s {
     gc_procs_common;
@@ -533,8 +533,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define CLEAR_MARKS_PROC(proc)\
   void proc(const gs_memory_t *cmem, void *vptr, uint size, const gs_memory_struct_type_t *pstype)
 
-	/* Complex structures with their own clear_marks, */
-	/* enum, reloc, and finalize procedures. */
+        /* Complex structures with their own clear_marks, */
+        /* enum, reloc, and finalize procedures. */
 
 #define gs__st_complex_only(scope_st, stname, stype, sname, pclear, penum, preloc, pfinal)\
   scope_st stname = { sizeof(stype), sname, 0, pclear, penum, preloc, pfinal, 0 }
@@ -554,7 +554,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_complex(stname, stype, sname, pclear, penum, preloc, pfinal)\
   gs__st_complex(private_st, stname, stype, sname, pclear, penum, preloc, pfinal)
 
-	/* Composite structures with their own enum and reloc procedures. */
+        /* Composite structures with their own enum and reloc procedures. */
 
 #define gs__st_composite(scope_st, stname, stype, sname, penum, preloc)\
   static struct_proc_enum_ptrs(penum);\
@@ -565,7 +565,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_composite(stname, stype, sname, penum, preloc)\
   gs__st_composite(private_st, stname, stype, sname, penum, preloc)
 
-	/* Composite structures with inherited finalization. */
+        /* Composite structures with inherited finalization. */
 
 #define gs__st_composite_use_final(scope_st, stname, stype, sname, penum, preloc, pfinal)\
   static struct_proc_enum_ptrs(penum);\
@@ -576,7 +576,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_composite_use_final(stname, stype, sname, penum, preloc, pfinal)\
   gs__st_composite_use_final(private_st, stname, stype, sname, penum, preloc, pfinal)
 
-	/* Composite structures with finalization. */
+        /* Composite structures with finalization. */
 
 #define gs__st_composite_final(scope_st, stname, stype, sname, penum, preloc, pfinal)\
   static struct_proc_finalize(pfinal);\
@@ -586,8 +586,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_composite_final(stname, stype, sname, penum, preloc, pfinal)\
   gs__st_composite_final(private_st, stname, stype, sname, penum, preloc, pfinal)
 
-	/* Composite structures with enum and reloc procedures */
-	/* already declared. */
+        /* Composite structures with enum and reloc procedures */
+        /* already declared. */
 
 #define gs__st_composite_only(scope_st, stname, stype, sname, penum, preloc)\
   gs__st_complex_only(scope_st, stname, stype, sname, 0, penum, preloc, 0)
@@ -598,9 +598,9 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 
 /* ---------------- Special kinds of structures ---------------- */
 
-	/* Element structures, for use in arrays of structures. */
-	/* Note that these require that the underlying structure's */
-	/* enum_ptrs procedure always return the same number of pointers. */
+        /* Element structures, for use in arrays of structures. */
+        /* Note that these require that the underlying structure's */
+        /* enum_ptrs procedure always return the same number of pointers. */
 
 #define gs__st_element(scope_st, stname, stype, sname, penum, preloc, basest)\
   static ENUM_PTRS_BEGIN_PROC(penum) {\
@@ -620,9 +620,27 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_element(stname, stype, sname, penum, preloc, basest)\
   gs__st_element(private_st, stname, stype, sname, penum, preloc, basest)
 
-	/* A "structure" just consisting of a pointer. */
-	/* Note that in this case only, stype is a pointer type. */
-	/* Fortunately, C's bizarre 'const' syntax does what we want here. */
+#define gs__st_element_final(scope_st, stname, stype, sname, penum, preloc, basest, pfinal)\
+  static ENUM_PTRS_BEGIN_PROC(penum) {\
+    uint count = size / (uint)sizeof(stype);\
+    if ( count == 0 ) return 0;\
+    return ENUM_USING(basest, (EV_CONST char *)vptr + (index % count) * sizeof(stype),\
+      sizeof(stype), index / count);\
+  } ENUM_PTRS_END_PROC\
+  static RELOC_PTRS_BEGIN(preloc) {\
+    uint count = size / (uint)sizeof(stype);\
+    for ( ; count; count--, vptr = (char *)vptr + sizeof(stype) )\
+      RELOC_USING(basest, vptr, sizeof(stype));\
+  } RELOC_PTRS_END\
+  gs__st_composite_final(scope_st, stname, stype, sname, penum, preloc,pfinal)
+#define gs_public_st_element_final(stname, stype, sname, penum, preloc, basest,pfinal)\
+  gs__st_element_final(public_st, stname, stype, sname, penum, preloc, basest,pfinal)
+#define gs_private_st_element_final(stname, stype, sname, penum, preloc, basest,pfinal)\
+  gs__st_element_final(private_st, stname, stype, sname, penum, preloc, basest,pfinal)
+
+        /* A "structure" just consisting of a pointer. */
+        /* Note that in this case only, stype is a pointer type. */
+        /* Fortunately, C's bizarre 'const' syntax does what we want here. */
 
 #define gs__st_ptr(scope_st, stname, stype, sname, penum, preloc)\
   static ENUM_PTRS_BEGIN(penum) return 0;\
@@ -640,7 +658,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 /* ---------- Ordinary structures with a fixed set of pointers ----------- */
 /* Note that we "cannibalize" the penum and preloc names for elts and sdata. */
 
-	/* Structures with 1 pointer. */
+        /* Structures with 1 pointer. */
 
 #define gs__st_ptrs1(scope_st, stname, stype, sname, penum, preloc, e1)\
   BASIC_PTRS(penum) {\
@@ -652,7 +670,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs1(stname, stype, sname, penum, preloc, e1)\
   gs__st_ptrs1(private_st, stname, stype, sname, penum, preloc, e1)
 
-	/* Structures with 1 string. */
+        /* Structures with 1 string. */
 
 #define gs__st_strings1(scope_st, stname, stype, sname, penum, preloc, e1)\
   BASIC_PTRS(penum) {\
@@ -664,7 +682,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_strings1(stname, stype, sname, penum, preloc, e1)\
   gs__st_strings1(private_st, stname, stype, sname, penum, preloc, e1)
 
-	/* Structures with 1 const string. */
+        /* Structures with 1 const string. */
 
 #define gs__st_const_strings1(scope_st, stname, stype, sname, penum, preloc, e1)\
   BASIC_PTRS(penum) {\
@@ -676,7 +694,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_const_strings1(stname, stype, sname, penum, preloc, e1)\
   gs__st_const_strings1(private_st, stname, stype, sname, penum, preloc, e1)
 
-	/* Structures with 2 const strings. */
+        /* Structures with 2 const strings. */
 
 #define gs__st_const_strings2(scope_st, stname, stype, sname, penum, preloc, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -688,7 +706,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_const_strings2(stname, stype, sname, penum, preloc, e1, e2)\
   gs__st_const_strings2(private_st, stname, stype, sname, penum, preloc, e1, e2)
 
-	/* Structures with 2 pointers. */
+        /* Structures with 2 pointers. */
 
 #define gs__st_ptrs2(scope_st, stname, stype, sname, penum, preloc, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -700,7 +718,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs2(stname, stype, sname, penum, preloc, e1, e2)\
   gs__st_ptrs2(private_st, stname, stype, sname, penum, preloc, e1, e2)
 
-	/* Structures with 3 pointers. */
+        /* Structures with 3 pointers. */
 
 #define gs__st_ptrs3(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3)\
   BASIC_PTRS(penum) {\
@@ -712,7 +730,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs3(stname, stype, sname, penum, preloc, e1, e2, e3)\
   gs__st_ptrs3(private_st, stname, stype, sname, penum, preloc, e1, e2, e3)
 
-	/* Structures with 4 pointers. */
+        /* Structures with 4 pointers. */
 
 #define gs__st_ptrs4(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4)\
   BASIC_PTRS(penum) {\
@@ -724,7 +742,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs4(stname, stype, sname, penum, preloc, e1, e2, e3, e4)\
   gs__st_ptrs4(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4)
 
-	/* Structures with 5 pointers. */
+        /* Structures with 5 pointers. */
 
 #define gs__st_ptrs5(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)\
   BASIC_PTRS(penum) {\
@@ -736,7 +754,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs5(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)\
   gs__st_ptrs5(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)
 
-	/* Structures with 6 pointers. */
+        /* Structures with 6 pointers. */
 
 #define gs__st_ptrs6(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6)\
   BASIC_PTRS(penum) {\
@@ -748,7 +766,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs6(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6)\
   gs__st_ptrs6(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6)
 
-	/* Structures with 7 pointers. */
+        /* Structures with 7 pointers. */
 
 #define gs__st_ptrs7(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7)\
   BASIC_PTRS(penum) {\
@@ -760,7 +778,55 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs7(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7)\
   gs__st_ptrs7(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7)
 
-	/* Structures with 1 const string and 1 pointer. */
+        /* Structures with 8 pointers. */
+
+#define gs__st_ptrs8(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3), GC_OBJ_ELT3(stype, e4, e5, e6), GC_OBJ_ELT2(stype, e7, e8)\
+  };\
+  gs__st_basic(scope_st, stname, stype, sname, penum, preloc)
+#define gs_public_st_ptrs8(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)\
+  gs__st_ptrs8(public_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)
+#define gs_private_st_ptrs8(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)\
+  gs__st_ptrs8(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)
+
+        /* Structures with 9 pointers. */
+
+#define gs__st_ptrs9(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3), GC_OBJ_ELT3(stype, e4, e5, e6), GC_OBJ_ELT3(stype, e7, e8, e9)\
+  };\
+  gs__st_basic(scope_st, stname, stype, sname, penum, preloc)
+#define gs_public_st_ptrs9(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9)\
+  gs__st_ptrs9(public_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9)
+#define gs_private_st_ptrs9(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9)\
+  gs__st_ptrs9(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9)
+
+        /* Structures with 10 pointers. */
+
+#define gs__st_ptrs10(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3), GC_OBJ_ELT3(stype, e4, e5, e6), GC_OBJ_ELT3(stype, e7, e8, e9),\
+  GC_OBJ_ELT(stype, e10) };\
+  gs__st_basic(scope_st, stname, stype, sname, penum, preloc)
+#define gs_public_st_ptrs10(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
+  gs__st_ptrs10(public_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)
+#define gs_private_st_ptrs10(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
+  gs__st_ptrs10(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)
+
+        /* Structures with 11 pointers. */
+
+#define gs__st_ptrs11(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  BASIC_PTRS(penum) {\
+    GC_OBJ_ELT3(stype, e1, e2, e3), GC_OBJ_ELT3(stype, e4, e5, e6), GC_OBJ_ELT3(stype, e7, e8, e9),\
+  GC_OBJ_ELT2(stype, e10, e11) };\
+  gs__st_basic(scope_st, stname, stype, sname, penum, preloc)
+#define gs_public_st_ptrs11(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  gs__st_ptrs11(public_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)
+#define gs_private_st_ptrs11(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
+  gs__st_ptrs11(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)
+
+        /* Structures with 1 const string and 1 pointer. */
 
 #define gs__st_const_strings1_ptrs1(scope_st, stname, stype, sname, penum, preloc, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -772,7 +838,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_const_strings1_ptrs1(stname, stype, sname, penum, preloc, e1, e2)\
   gs__st_const_strings1_ptrs1(private_st, stname, stype, sname, penum, preloc, e1, e2)
 
-	/* Structures with 1 const string and 4 pointers. */
+        /* Structures with 1 const string and 4 pointers. */
 
 #define gs__st_strings1_ptrs4(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)\
   BASIC_PTRS(penum) {\
@@ -785,7 +851,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_strings1_ptrs4(stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)\
   gs__st_strings1_ptrs4(private_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5)
 
-	/* Structures with 1 const string and 7 pointers. */
+        /* Structures with 1 const string and 7 pointers. */
 
 #define gs__st_strings1_ptrs7(scope_st, stname, stype, sname, penum, preloc, e1, e2, e3, e4, e5, e6, e7, e8)\
   BASIC_PTRS(penum) {\
@@ -800,7 +866,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 
 /* ---------------- Suffix subclasses ---------------- */
 
-	/* Suffix subclasses with no additional pointers. */
+        /* Suffix subclasses with no additional pointers. */
 
 #define gs__st_suffix_add0(scope_st, stname, stype, sname, penum, preloc, supstname)\
   gs__st_basic_with_super_final(scope_st, stname, stype, sname, 0, 0, preloc, &supstname, 0, 0)
@@ -809,10 +875,10 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add0(stname, stype, sname, penum, preloc, supstname)\
   gs__st_suffix_add0(private_st, stname, stype, sname, penum, preloc, supstname)
 
-	/* Suffix subclasses with no additional pointers, */
-	/* and with the superclass defined earlier in the same file */
-	/* as a 'basic' type. */
-	/* In this case, we don't even need new procedures. */
+        /* Suffix subclasses with no additional pointers, */
+        /* and with the superclass defined earlier in the same file */
+        /* as a 'basic' type. */
+        /* In this case, we don't even need new procedures. */
 
 #define gs__st_suffix_add0_local(scope_st, stname, stype, sname, supenum, supreloc, supstname)\
   scope_st stname = {\
@@ -824,10 +890,10 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add0_local(stname, stype, sname, supenum, supreloc, supstname)\
   gs__st_suffix_add0_local(private_st, stname, stype, sname, supenum, supreloc, supstname)
 
-	/* Suffix subclasses with no additional pointers and finalization. */
-	/* This is a hack -- subclasses should inherit finalization, */
-	/* but that would require a superclass pointer in the descriptor, */
-	/* which would perturb things too much right now. */
+        /* Suffix subclasses with no additional pointers and finalization. */
+        /* This is a hack -- subclasses should inherit finalization, */
+        /* but that would require a superclass pointer in the descriptor, */
+        /* which would perturb things too much right now. */
 
 #define gs__st_suffix_add0_final(scope_st, stname, stype, sname, penum, preloc, pfinal, supstname)\
   static ENUM_PTRS_BEGIN_PROC(penum) {\
@@ -842,7 +908,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add0_final(stname, stype, sname, penum, preloc, pfinal, supstname)\
   gs__st_suffix_add0_final(private_st, stname, stype, sname, penum, preloc, pfinal, supstname)
 
-	/* Suffix subclasses with 1 additional pointer. */
+        /* Suffix subclasses with 1 additional pointer. */
 
 #define gs__st_suffix_add1(scope_st, stname, stype, sname, penum, preloc, supstname, e1)\
   BASIC_PTRS(penum) {\
@@ -854,8 +920,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add1(stname, stype, sname, penum, preloc, supstname, e1)\
   gs__st_suffix_add1(private_st, stname, stype, sname, penum, preloc, supstname, e1)
 
-	/* Suffix subclasses with 1 additional pointer and finalization. */
-	/* See above regarding finalization and subclasses. */
+        /* Suffix subclasses with 1 additional pointer and finalization. */
+        /* See above regarding finalization and subclasses. */
 
 #define gs__st_suffix_add1_final(scope_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1)\
   BASIC_PTRS(penum) {\
@@ -867,7 +933,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add1_final(stname, stype, sname, penum, preloc, pfinal, supstname, e1)\
   gs__st_suffix_add1_final(private_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1)
 
-	/* Suffix subclasses with 1 additional string. */
+        /* Suffix subclasses with 1 additional string. */
 
 #define gs__st_suffix_add_strings1(scope_st, stname, stype, sname, penum, preloc, supstname, e1)\
   BASIC_PTRS(penum) {\
@@ -879,7 +945,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add_strings1(stname, stype, sname, penum, preloc, supstname, e1)\
   gs__st_suffix_add_strings1(private_st, stname, stype, sname, penum, preloc, supstname, e1)
 
-	/* Suffix subclasses with 2 additional pointers. */
+        /* Suffix subclasses with 2 additional pointers. */
 
 #define gs__st_suffix_add2(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -891,7 +957,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add2(stname, stype, sname, penum, preloc, supstname, e1, e2)\
   gs__st_suffix_add2(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2)
 
-	/* Suffix subclasses with 1 additional pointers and 1 string. */
+        /* Suffix subclasses with 1 additional pointers and 1 string. */
 
 #define gs__st_suffix_add1_string1(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -903,7 +969,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
   gs__st_suffix_add1_string1(public_st, stname, stype, sname, penum, preloc, supstname, e1, e2)
 #define gs_private_st_suffix_add1_string1(stname, stype, sname, penum, preloc, supstname, e1, e2)\
   gs__st_suffix_add1_string1(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2)
-	/* Suffix subclasses with 2 additional pointers and 1 string. */
+        /* Suffix subclasses with 2 additional pointers and 1 string. */
 
 #define gs__st_suffix_add2_string1(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
   BASIC_PTRS(penum) {\
@@ -916,8 +982,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add2_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
   gs__st_suffix_add2_string1(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)
 
-	/* Suffix subclasses with 2 additional pointers and finalization. */
-	/* See above regarding finalization and subclasses. */
+        /* Suffix subclasses with 2 additional pointers and finalization. */
+        /* See above regarding finalization and subclasses. */
 
 #define gs__st_suffix_add2_final(scope_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2)\
   BASIC_PTRS(penum) {\
@@ -940,7 +1006,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_string2_final(stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2)\
   gs__st_suffix_string2_final(private_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2)
 
-	/* Suffix subclasses with 3 additional pointers. */
+        /* Suffix subclasses with 3 additional pointers. */
 
 #define gs__st_suffix_add3(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
   BASIC_PTRS(penum) {\
@@ -952,7 +1018,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add3(stname, stype, sname, penum, preloc, supstname, e1, e2, e3)\
   gs__st_suffix_add3(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3)
 
-	/* Suffix subclasses with 3 additional pointers and 1 string. */
+        /* Suffix subclasses with 3 additional pointers and 1 string. */
 
 #define gs__st_suffix_add3_string1(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
   BASIC_PTRS(penum) {\
@@ -965,8 +1031,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add3_string1(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
   gs__st_suffix_add3_string1(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)
 
-	/* Suffix subclasses with 3 additional pointers and finalization. */
-	/* See above regarding finalization and subclasses. */
+        /* Suffix subclasses with 3 additional pointers and finalization. */
+        /* See above regarding finalization and subclasses. */
 
 #define gs__st_suffix_add3_final(scope_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3)\
   BASIC_PTRS(penum) {\
@@ -978,7 +1044,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add3_final(stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3)\
   gs__st_suffix_add3_final(private_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3)
 
-	/* Suffix subclasses with 4 additional pointers. */
+        /* Suffix subclasses with 4 additional pointers. */
 
 #define gs__st_suffix_add4(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
   BASIC_PTRS(penum) {\
@@ -990,8 +1056,8 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add4(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)\
   gs__st_suffix_add4(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4)
 
-	/* Suffix subclasses with 4 additional pointers and finalization. */
-	/* See above regarding finalization and subclasses. */
+        /* Suffix subclasses with 4 additional pointers and finalization. */
+        /* See above regarding finalization and subclasses. */
 
 #define gs__st_suffix_add4_final(scope_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3, e4)\
   BASIC_PTRS(penum) {\
@@ -1003,7 +1069,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add4_final(stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3, e4)\
   gs__st_suffix_add4_final(private_st, stname, stype, sname, penum, preloc, pfinal, supstname, e1, e2, e3, e4)
 
-	/* Suffix subclasses with 5 additional pointers. */
+        /* Suffix subclasses with 5 additional pointers. */
 
 #define gs__st_suffix_add5(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5)\
   BASIC_PTRS(penum) {\
@@ -1015,7 +1081,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add5(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5)\
   gs__st_suffix_add5(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5)
 
-	/* Suffix subclasses with 6 additional pointers. */
+        /* Suffix subclasses with 6 additional pointers. */
 
 #define gs__st_suffix_add6(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6)\
   BASIC_PTRS(penum) {\
@@ -1027,7 +1093,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add6(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6)\
   gs__st_suffix_add6(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6)
 
-	/* Suffix subclasses with 7 additional pointers. */
+        /* Suffix subclasses with 7 additional pointers. */
 
 #define gs__st_suffix_add7(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7)\
   BASIC_PTRS(penum) {\
@@ -1040,7 +1106,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add7(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7)\
   gs__st_suffix_add7(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7)
 
-	/* Suffix subclasses with 8 additional pointers. */
+        /* Suffix subclasses with 8 additional pointers. */
 
 #define gs__st_suffix_add8(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8)\
   BASIC_PTRS(penum) {\
@@ -1053,7 +1119,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add8(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8)\
   gs__st_suffix_add8(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8)
 
-	/* Suffix subclasses with 9 additional pointers. */
+        /* Suffix subclasses with 9 additional pointers. */
 
 #define gs__st_suffix_add9(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9)\
   BASIC_PTRS(penum) {\
@@ -1066,7 +1132,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add9(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9)\
   gs__st_suffix_add9(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9)
 
-	/* Suffix subclasses with 10 additional pointers. */
+        /* Suffix subclasses with 10 additional pointers. */
 
 #define gs__st_suffix_add10(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
   BASIC_PTRS(penum) {\
@@ -1079,7 +1145,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_suffix_add10(stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)\
   gs__st_suffix_add10(private_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10)
 
-	/* Suffix subclasses with 11 additional pointers. */
+        /* Suffix subclasses with 11 additional pointers. */
 
 #define gs__st_suffix_add11(scope_st, stname, stype, sname, penum, preloc, supstname, e1, e2, e3, e4, e5, e6, e7, e8, e9, e10, e11)\
   BASIC_PTRS(penum) {\
@@ -1094,7 +1160,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 
 /* ---------------- General subclasses ---------------- */
 
-	/* General subclasses with no additional pointers. */
+        /* General subclasses with no additional pointers. */
 
 #define gs__st_ptrs_add0(scope_st, stname, stype, sname, penum, preloc, supstname, member)\
   gs__st_basic_with_super_final(scope_st, stname, stype, sname, 0, 0, preloc, &supstname, offset_of(stype, member), 0)
@@ -1103,7 +1169,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs_add0(stname, stype, sname, penum, preloc, supstname, member)\
   gs__st_ptrs_add0(private_st, stname, stype, sname, penum, preloc, supstname, member)
 
-	/* General subclasses with 1 additional pointer. */
+        /* General subclasses with 1 additional pointer. */
 
 #define gs__st_ptrs_add1(scope_st, stname, stype, sname, penum, preloc, supstname, member, e1)\
   BASIC_PTRS(penum) {\
@@ -1115,7 +1181,7 @@ extern void reloc_const_bytestring(gs_const_bytestring *pbs, gc_state_t *gcst);
 #define gs_private_st_ptrs_add1(stname, stype, sname, penum, preloc, supstname, member, e1)\
   gs__st_ptrs_add1(private_st, stname, stype, sname, penum, preloc, supstname, member, e1)
 
-	/* General subclasses with 2 additional pointers. */
+        /* General subclasses with 2 additional pointers. */
 
 #define gs__st_ptrs_add2(scope_st, stname, stype, sname, penum, preloc, supstname, member, e1, e2)\
   BASIC_PTRS(penum) {\
